@@ -30,13 +30,16 @@ Detecting sleep mode
 """
 
 import os.path
-import sys
 import pywintypes
+import sys
+import time
 from datetime import datetime
 from locale import getpreferredencoding
+from threading import Thread, Event
+
 from six import text_type, binary_type, string_types, PY2
+
 from ..base        import EngineBase, EngineError, MimicFailure
-from ...error import GrammarError
 from .dictation    import NatlinkDictationContainer
 from .recobs       import NatlinkRecObsManager
 from .timer        import NatlinkTimerManager
@@ -60,6 +63,42 @@ def map_word(word, encoding=getpreferredencoding(do_setlocale=False)):
     return word
 
 
+class TimerThread(Thread):
+    """"""
+    def __init__(self, engine):
+        Thread.__init__(self)
+        self._stop_event = Event()
+        self.daemon = True
+        self._timer = None
+        self._engine = engine
+
+    def start(self):
+        if self._timer is None:
+            def timer_function():
+                # Let the thread run for a bit. This will yield control to
+                # other threads.
+                if self.is_alive():
+                    self.join(0.0025)
+
+            self._timer = self._engine.create_timer(timer_function, 0.025)
+
+        Thread.start(self)
+
+    def _stop_timer(self):
+        if self._timer:
+            self._timer.stop()
+            self._timer = None
+
+    def stop(self):
+        self._stop_event.set()
+        self._stop_timer()
+
+    def run(self):
+        while not self._stop_event.is_set():
+            time.sleep(1)
+        self._stop_timer()
+
+
 class NatlinkEngine(EngineBase):
     """ Speech recognition engine back-end for Natlink and DNS. """
 
@@ -82,6 +121,7 @@ class NatlinkEngine(EngineBase):
         self._grammar_count = 0
         self._recognition_observer_manager = NatlinkRecObsManager(self)
         self._timer_manager = NatlinkTimerManager(0.02, self)
+        self._timer_thread = None
         self._retain_dir = None
         try:
             self.set_retain_directory(retain_dir)
@@ -89,9 +129,26 @@ class NatlinkEngine(EngineBase):
             self._retain_dir = None
             self._log.error(err)
 
+    def apply_threading_fix(self):
+        """
+        Start a thread and engine timer internally to allow Python threads
+        to work properly while connected to natlink. The fix is only applied
+        once, successive calls have no effect.
+
+        This method is called automatically when :meth:`connect` is called
+        or when a grammar is loaded for the first time.
+        """
+        # Start a thread and engine timer to allow Python threads to work
+        # properly while connected to Natlink.
+        # Only start the thread if one isn't already active.
+        if self._timer_thread is None:
+            self._timer_thread = TimerThread(self)
+            self._timer_thread.start()
+
     def connect(self):
         """ Connect to natlink with Python threading support enabled. """
         self.natlink.natConnect(True)
+        self.apply_threading_fix()
 
     def disconnect(self):
         """ Disconnect from natlink. """
@@ -111,6 +168,11 @@ class NatlinkEngine(EngineBase):
                     pass
                 break
 
+        # Stop the special timer thread if it is running.
+        if self._timer_thread:
+            self._timer_thread.stop()
+            self._timer_thread = None
+
         # Finally disconnect from natlink.
         self.natlink.natDisconnect()
 
@@ -161,6 +223,10 @@ class NatlinkEngine(EngineBase):
                 raise EngineError("Failed to load grammar %s: %s."
                                   % (grammar, e))
 
+        # Apply the threading fix if it hasn't been applied yet.
+        self.apply_threading_fix()
+
+        # Return the grammar wrapper.
         return wrapper
 
     def _unload_grammar(self, grammar, wrapper):
@@ -273,41 +339,8 @@ class NatlinkEngine(EngineBase):
         app = win32com.client.Dispatch("Dragon.DgnEngineControl")
         language = app.SpeakerLanguage("")
 
-        # Lookup the language tags.
-        tags = self._language_tags.get(language)
-        if tags:
-            return tags[0]
-
-        # The _language_tags dictionary didn't contain the language, so
-        # get the best match by using the primary language identifier.
-        # This allows us to match unlisted language variants.
-        primary_id = language & 0x00ff
-        for lang_id, (tag, _) in self._language_tags.items():
-            if primary_id == lang_id & 0x00ff:  # Match found.
-                return tag
-
-        # Speaker language wasn't found.
-        self._log.error("Unknown speaker language: 0x%04x" % language)
-        raise GrammarError("Unknown speaker language: 0x%04x" % language)
-
-    _language_tags = {
-                      0x0c09: ("en", "AustralianEnglish"),
-                      0xf00a: ("es", "CastilianSpanish"),
-                      0x0413: ("nl", "Dutch"),
-                      0x0009: ("en", "English"),
-                      0x040c: ("fr", "French"),
-                      0x0407: ("de", "German"),
-                      0xf009: ("en", "IndianEnglish"),
-                      0x0410: ("it", "Italian"),
-                      0x0411: ("jp", "Japanese"),
-                      0xf40a: ("es", "LatinAmericanSpanish"),
-                      0x0416: ("pt", "Portuguese"),
-                      0xf409: ("en", "SingaporeanEnglish"),
-                      0x040a: ("es", "Spanish"),
-                      0x0809: ("en", "UKEnglish"),
-                      0x0409: ("en", "USEnglish"),
-                      0xf809: ("en", "CAEnglish"),
-                     }
+        # Lookup and return the language tag.
+        return self._get_language_tag(language)
 
     def set_retain_directory(self, retain_dir):
         """
